/**
 * How far beyond the currently displayed image to buffer.
 *
 * @type {number}
 */
const carouselBufferSize = 1;

const properties = {
  sizesBackup: 'originalSizes',
  resizeFlag: 'hasEventToUpdateFullscreenOnResize',
};

const classes = {
  opener: 'opener',
  opened: 'opener-opened',
  image: 'opener-image',
  container: 'figure-img-container',
  carousel: 'carousel',
  item: 'carousel-item',
  modal: 'modal-open',
  activeItem: 'active',
};

const selectors = {
  body:        'body',
  opener:      '.' + classes.opener,
  opened:      '.' + classes.opened,
  image:       '.' + classes.image + ' img',
  container:   '.' + classes.container,
  carousel:    '.' + classes.carousel,
  item:        '.' + classes.item,
  buttonOpen:  '.' + classes.opener + ' .opener-button.opener-button-open, .' + classes.opener + ' .' + classes.image + ' img',
  buttonClose: '.' + classes.opener + ' .opener-button.opener-button-close',
};

const $buttonOpen = $(selectors.buttonOpen);

const $buttonClose = $(selectors.buttonClose);

const $carousels = $(selectors.carousel);

/**
 * @param {jQuery} $container
 */
const findImage = function($container) {
  return $container.find(selectors.image);
};

/**
 * If `index` goes outside of the boundaries of 0 and `length`-1, loop it back in.
 *
 * @param {number} index
 * @param {number} length
 * @returns {number}
 */
const advanceInLoop = function (index, length) {
  index = index % length;
  return (index < 0) ? index + length : index
};

/**
 * If image opener is part of a carousel, choose the carousel.
 *
 * @param {jQuery} $opener
 * @returns {jQuery}
 */
const assignOpener = function($opener) {
  const $carousel = $opener.closest(selectors.carousel);

  return ($carousel.length > 0) ? $carousel : $opener;
};

/**
 * Whether this element is the child of an opener that is opened.
 *
 * @param {jQuery} $el
 * @returns {boolean}
 */
const isOpened = function($el) {
  return ($el.closest(selectors.opened).length > 0);
};

/**
 * Whether this element is the child of a carousel item that is inactive.
 *
 * @param {jQuery} $el
 * @returns {boolean}
 */
const isInInactiveCarouselItem = function($el) {
  const $item = $el.closest(selectors.item);

  return ($item.length > 0) ? ! $item.hasClass(classes.activeItem) : false;
};

/**
 * @param {jQuery} $canvas Element which the image is scaled to fit inside.
 * @param {jQuery} $image The image to be scaled.
 *
 * @returns {number} Percentage of viewport width
 */
const getFullscreenImageWidth = function($canvas, $image) {
  const
    canvasRatio = $canvas.width() / $canvas.height(),
    imageRatio = ($image.attr('width') > 0 && $image.attr('height') > 0)
      ? $image.attr('width') / $image.attr('height')
      : $image[0].naturalWidth / $image[0].naturalHeight,
    canvasFractionOfViewport = $canvas.width() / $(window).width();

  // Width as fraction of canvas width. If image is more portrait-sized than the canvas, take the height into consideration.
  const maxWidth = (imageRatio < canvasRatio) ? imageRatio / canvasRatio : 1.0;

  return maxWidth * canvasFractionOfViewport * 100;
};

/**
 * @param {jQuery} $container
 * @param {jQuery|undefined} $image The image to be scaled. If undefined, get image from $container instead.
 */
const applyFullscreenResolution = function ($container, $image = undefined) {
  if ($image === undefined) {
    $image = findImage($container);
  }

  // Backup original value before changing.
  if ($image.data(properties.sizesBackup) === undefined) {
    $image.data(properties.sizesBackup, $image.attr('sizes'));
  }

  const $canvas = $container.find(selectors.container);

  $image.attr('sizes', getFullscreenImageWidth($canvas, $image).toFixed(2) + 'vw');

  // Add event to update on resize. Only add once per image!
  if ($image.data(properties.resizeFlag) === true) {
    return;
  }
  $(window).resize({image: $image}, function(e) {
    const $image = e.data.image;
    if (isOpened($image) && !isInInactiveCarouselItem($image)) {
      applyFullscreenResolution($image.closest(selectors.opener));
    }
  });
  $image.data(properties.resizeFlag, true);
};

/**
 * @param {jQuery} $container
 */
const removeFullscreenResolution = function ($container) {
  const $image = findImage($container);

  // Restore backup:ed value.
  $image.attr('sizes', $image.data(properties.sizesBackup));
  $image.data(properties.sizesBackup, undefined);
};

/**
 * @param {Event} e
 */
const applyFullscreenOnCarouselSlide = function (e) {
  if (!isOpened($(e.target))) {
    return;
  }

  const $items = $(e.target).find(selectors.item);

  /**
   * 1 for sliding forward (right), -1 for sliding backward (left).
   *
   * @type {number}
   */
  const slideDirection = (e.from + 1 === e.to || (e.to === 0 && e.from !== 1)) ? 1 : -1;

  for (let steps = 1; steps - 1 <= carouselBufferSize; steps++) {
    const
      currentIndex = e.from,
      destinationIndex = advanceInLoop(currentIndex + (steps * slideDirection), $items.length),
      $container = $items.eq(currentIndex),
      $image = findImage($items.eq(destinationIndex));

    /**
     * $image may be an image currently off-screen, but $container needs to belong to the image currently on-screen.
     * This is because the container of an off-screen image has no physical dimensions yet.
     */
    applyFullscreenResolution($container, $image);
  }
};

// Image opener
$buttonOpen.click(function () {
  const $opener = $(this).closest('.opener');

  $(selectors.body).addClass(classes.modal);
  assignOpener($opener).addClass(classes.opened);
  applyFullscreenResolution($opener);
});

$buttonClose.click(function () {
  const $opener = $(this).closest('.opener');

  $(selectors.body).removeClass(classes.modal);
  assignOpener($opener).removeClass(classes.opened);
  removeFullscreenResolution($opener);
});

$(document).keydown(function (e) {
  if (e.keyCode === 27) {
    $buttonClose.click();
  }
});

$carousels.on('slide.bs.carousel', applyFullscreenOnCarouselSlide);
