<?php

namespace EuKit\Base\ViewModels;

use EuKit\Base\Parts\NewsMLArticle;
use EuKit\Base\Parts\Image;
use EuKit\Base\Parts\Concept;
use EuKit\Base\Parts\ArticleBodyPresentation;
use EuKit\Base\SiteLayout;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Base\ThemeMods;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Tightenco\Collect\Support\Collection;

class ArticlePage extends Page
{
    public function __construct(NewsMLArticle $article)
    {
        $args = [
            'max_cols'      => SiteLayout::getContentSizeGridMaxColumns(),
            'content_sizes' => SiteLayout::getContentSizes('article'),
        ];

        $articleBody = $article->getParsedBody();
        $articleBodyPresentation = new ArticleBodyPresentation($article, $args);

        $socialTeaser = $article->getNewsMLSocialTeaser($article);
        $socialTeaser['text'] = !empty($socialTeaser['text']) ? strip_tags($socialTeaser['text']) : '';

        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill($article->getProperties());
        $propertyObject->fill([
            'teaserImage' => $this->getTeaserImage($article),
            'socialTeaser' => $socialTeaser,
            'socialTeaserImage' => $this->getSocialTeaserImage($article),
            'body' => $articleBody->generateContent($articleBodyPresentation),
            'relatedArticles' => $articleBody->allOfType('x-im/articlex'),
            'settings' => ThemeMods::getModsByPrefix('article'),
        ]);

        $propertyObject->fill($this->getConceptData($article));
        parent::__construct($propertyObject);
    }

    private function getTeaserImageViewData($teaser) {
        $image = $teaser ? $teaser->getImage() : null;

        if ($image !== null) {
            $image->set('text', $teaser->title);
            $image = new Image($image);
        }

        return $image ? $image->getViewData() : null;
    }

    protected function getTeaserImage(NewsMLArticle $article)
    {
        return $this->getTeaserImageViewData($article->getNewsMLTeaser());
    }

    protected function getSocialTeaserImage(NewsMLArticle $article)
    {
        return $this->getTeaserImageViewData($article->getNewsMLSocialTeaser());
    }

    /**
     * Related Articles
     *
     * Add other necessary data to viewdata.
     * Add Related articles from matching concept.
     *
     * @param NewsMLArticle $article
     * @return void
     */

    protected function getRelatedArticles(NewsMLArticle $article): void
    {
      $currentDate = gmdate("Y-m-d\TH:i:s\Z");
      $startDate = gmdate("Y-m-d\TH:i:s\Z", strtotime( '-300 week'));
      $dateSpan = '[' . $startDate . ' TO ' . $currentDate . ']';

      $provider = OpenContentProvider::setup([
        'q' => QueryBuilder::where('Channels', $article->Channels)
          ->andIfProperty('ConceptUuids', $article->ConceptTagUuids[0] ?? $article->ConceptCategoryUuids[0] ?? $article->ConceptSectionUuids[0] ?? '')
          ->andIfNotProperty('uuid', $article->uuid)
          ->buildQueryString(),
        'contenttypes'           => [ 'Article' ],
        'Pubdate'                => $dateSpan,
        'sort.indexfield'        => 'Pubdate',
        'sort.Pubdate.ascending' => 'false',
        'limit'                  => 6
      ]);

      $provider->setPropertyMap('Article');

      $articles = array_map(static function (NewsMLArticle $article) {
        return (new Teaser($article))->getViewData();
      }, $provider->queryWithRequirements());

      $this->setViewData('related_articles', $articles);
    }

    private function getConceptData(NewsMLArticle $article): array
    {
        return [
            'authors' => $this->extractViewData($article->getAuthors()),
            'tags' => $this->extractViewData($article->getTags()),
            'places' => $this->extractViewData($article->getFilteredPlaces()),
            'categories' => $this->extractViewData($article->getCategories()),
            'channels' => $this->extractViewData($article->getChannels()),
            'stories' => $this->extractViewData($article->getStories())
        ];
    }

    private function extractViewData(Collection $collection)
    {
        return $collection->map(function (Concept $concept) {
            return (new RelatedConcept($concept))->getViewData();
        });
    }
}
